/***************************************************************************
                          agent.cpp  -  description
                             -------------------
    begin                : Sun Sep 15 2002
    copyright            : (C) 2002 by Michal Ramsza
    email                : mramsz@sgh.waw.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "agent.h"
#include "random.h"


//default constructor
agent::agent(void)
{
  SamplingProbability=0.0;
  K=1; // 1-sampling by default
  A=pf_2point(.5); // default random initial sitution
  T=0;
  AW=0;
  Payoff[0]=0; Payoff[1]=0;
  Status[0]=0; Status[1]=0; Status[2]=0;// all passive by default
}

void agent::preset(float p, int k, float ip)
{
  SamplingProbability=p;
  K=k;
  A=pf_2point(1-ip); // random initial sitution
  T=0;
  AW=0;
  Payoff[0]=0; Payoff[1]=0;
  Status[0]=0; Status[1]=0; Status[2]=0;// all passive
}

agent::~agent()
{
  // nothing yet
}

int agent::payoff(float x) // getting a payoff thru this
{
  AW = (AW*(T-1.0) + x)/T; // updating average payoff
  if(Status[0]==1) Payoff[Status[1]] += x; // updating sampling payoffs
  return 0;
}

int agent::play(void) // sending an action thru this
{
  T++; // updating number of times played
  if(Status[0]==1)
  {
    if(Status[2]==K)
    {
      if(Status[1]==1)
      {
        // everything has been tested
        if(pf_2point(SamplingProbability))
        { // want more testing
          Status[0]=1; // starting sampling procedure
          Status[1]=0; // first zero-action
          Status[2]=1; // first sample
          A=0;
        } else{
          // not testing anymore
          Status[0]=0; // reseting status indictor
          // which action ?
          if(Payoff[0]>Payoff[1]) A=0;
          if(Payoff[0]<Payoff[1]) A=1;
          if(Payoff[0]==Payoff[1]) A=pf_2point(.5);// random choice
          Payoff[0]=0; Payoff[1]=0; // reseting sample payoff
        }
      } else {
        Status[1]=1;// next/last action
        Status[2]=1; // first sample
        A=1;
      }
    } else {
      Status[2]++; // next sample, dont change action A
    }
  } else {
    if(pf_2point(SamplingProbability))
    {
      Status[0]=1; // starting sampling procedure
      Status[1]=0; // first zero-action
      Status[2]=1; // first sample
      A=0;
    }
  }
  return A;
}

int agent::ReportStatus(void) // reporting status
{
  return Status[0];
}


int agent::ReportAction(void) // reporting action used
{
  return A;
}

float agent::ReportWealth(void) // reporting average wealth
{
  return AW;
}

float agent::ReportTime(void) // reporting how may times he played (used to compute frequency - testing)
{
  return T;
}

