/***************************************************************************
                          world.cpp  -  description
                             -------------------
    begin                : Sun Sep 15 2002
    copyright            : (C) 2002 by Michal Ramsza
    email                : mramsz@sgh.waw.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "world.h"
#include "configure.h"
#include "random.h"


world::world()
{
  int i;
  /* opening config file */
  if( ((config_file = fopen("./sampling.ini","r"))) == NULL){
    printf("Can't open config file. Exiting ...\n");
    exit(1); }

  /* reading parameters from config file */
  configure(config_file, "G_IniProb", "float", (void *) &G_IniProb);
  configure(config_file, "G_NumOfAgents", "int", (void *) &G_NumOfAgents);
  configure(config_file, "G_DataFile", "string", (void *) &G_DataFile);
  configure(config_file, "G_DescFile", "string", (void *) &G_DescFile);
  configure(config_file, "G_NumOfSteps", "int", (void *) &G_NumOfSteps);
  configure(config_file, "G_SamplingProbability", "float", (void *) &G_SamplingProbability);
  configure(config_file, "G_SamplingLength", "int", (void *) &G_SamplingLength);
  configure(config_file, "G_Step", "int", (void *) &G_Step);
  configure(config_file, "G_PayoffA", "float", (void *) &Payoff[0][0]);
  configure(config_file, "G_PayoffB", "float", (void *) &Payoff[0][1]);
  configure(config_file, "G_PayoffC", "float", (void *) &Payoff[1][0]);
  configure(config_file, "G_PayoffD", "float", (void *) &Payoff[1][1]);
  configure(config_file, "S_Active", "int", (void *) &S_Active);
  configure(config_file, "S_ActionActive", "int", (void *) &S_ActionActive);
  configure(config_file, "S_Passive", "int", (void *) &S_Passive);
  configure(config_file, "S_ActionPassive", "int", (void *) &S_ActionPassive);
  configure(config_file, "S_AverageWealth", "int", (void *) &S_AverageWealth);


  // closing config file
   fclose(config_file);

  /* opening output data file */
  if( (data_file = fopen(G_DataFile, "w")) == NULL){
    printf("Can't open data file. Exiting ...\n");
    exit(1); }

  /* opening output data file for averages */
  if( (desc_file = fopen(G_DescFile, "w")) == NULL){
    printf("Can't open desc. file. Exiting ...\n");
    exit(1); }

  time=0;

  // creating system of agents
  A = (agent *) new agent[G_NumOfAgents]; // this is the better way
  for(i=0; i<G_NumOfAgents; i++)
  {
    A[i].preset(G_SamplingProbability,G_SamplingLength,G_IniProb);
  }
}

world::~world()
{


  // removing system of agents
   delete [] A;
  // closing data files
  fclose(data_file);
  fclose(desc_file);
}


int world::step(void)
{
  int t;
  int player0, player1;
  int act0, act1;

  for(t=0; (t<G_Step)&&(time<G_NumOfSteps); t++)
  {
    time++;
    player0=pf_uniformk(G_NumOfAgents);
    player1=player0;
    while(player1==player0) player1=pf_uniformk(G_NumOfAgents);
    act0=A[player0].play();
    act1=A[player1].play();
    A[player0].payoff(Payoff[act0][act1]);
    A[player1].payoff(Payoff[act1][act0]);
  }
  if(!(time<G_NumOfSteps)) return 0;
  return 1;
}

int world::prereport(void)
{
  fprintf(desc_file, "Time\n");
  if(S_ActionActive) fprintf(desc_file, "Number of Active players using first strategy\n");
  if(S_ActionPassive) fprintf(desc_file, "Number of Passive players using first strategy\n");
  if(S_Active) fprintf(desc_file, "Number of Active players\n");
  if(S_Passive) fprintf(desc_file, "Number of Passive players\n");
  if(S_AverageWealth) fprintf(desc_file, "Time average of payoff per capita\n");

  return 0;
}

int world::report(void)
{
  int i;
  int n=0;
  float x=0;

  fprintf(data_file, "%10d ", time);
  if(S_ActionActive){
    for(i=0; i<G_NumOfAgents; i++){
      if((A[i].ReportStatus()==1)&&(A[i].ReportAction()==0)) n++;
    }
    fprintf(data_file, "%6d ", n);
    n=0;
  }

  if(S_ActionPassive){
    for(i=0; i<G_NumOfAgents; i++){
     if((A[i].ReportStatus()==0)&&(A[i].ReportAction()==0)) n++;
   }
   fprintf(data_file, "%6d ", n);
   n=0;
  }

  if(S_Active){
    for(i=0; i<G_NumOfAgents; i++){
      if(A[i].ReportStatus()==1) n++;
    }
    fprintf(data_file, "%6d ", n);
    n=0;
  }

  if(S_Passive){
    for(i=0; i<G_NumOfAgents; i++){
      if(A[i].ReportStatus()==0) n++;
    }
    fprintf(data_file, "%6d ", n);
    n=0;
  }

  if(S_AverageWealth){
    x=0;
    for(i=0; i<G_NumOfAgents; i++){
      x += A[i].ReportWealth();
    }
    x = x/(float)G_NumOfAgents;
    fprintf(data_file, "%9.3f ", x);
    x=0;
  }

  fprintf(data_file,"\n");
  return 0;
}

